<?php

namespace Assetdocs\Incident\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Assetdocs\Incident\Http\Requests\IncidentAdminApiRequest;
use Assetdocs\Incident\Interfaces\IncidentRepositoryInterface;
use Assetdocs\Incident\Models\Incident;

/**
 * Admin API controller class.
 */
class IncidentAdminApiController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'admin.api';

    /**
     * Initialize incident controller.
     *
     * @param type IncidentRepositoryInterface $incident
     *
     * @return type
     */
    public function __construct(IncidentRepositoryInterface $incident)
    {
        $this->middleware('api');
        $this->middleware('jwt.auth:admin.api');
        $this->setupTheme(config('theme.themes.admin.theme'), config('theme.themes.admin.layout'));

        $this->repository = $incident;
        parent::__construct();
    }

    /**
     * Display a list of incident.
     *
     * @return json
     */
    public function index(IncidentAdminApiRequest $request)
    {
        $incidents  = $this->repository
            ->pushCriteria(new \Assetdocs\Incident\Repositories\Criteria\IncidentAdminCriteria())
            ->setPresenter('\\Assetdocs\\Incident\\Repositories\\Presenter\\IncidentListPresenter')
            ->scopeQuery(function($query){
                return $query->orderBy('id','DESC');
            })->all();
        $incidents['code'] = 2000;
        return response()->json($incidents) 
                         ->setStatusCode(200, 'INDEX_SUCCESS');

    }

    /**
     * Display incident.
     *
     * @param Request $request
     * @param Model   Incident
     *
     * @return Json
     */
    public function show(IncidentAdminApiRequest $request, Incident $incident)
    {
        $incident         = $incident->presenter();
        $incident['code'] = 2001;
        return response()->json($incident)
                         ->setStatusCode(200, 'SHOW_SUCCESS');;

    }

    /**
     * Show the form for creating a new incident.
     *
     * @param Request $request
     *
     * @return json
     */
    public function create(IncidentAdminApiRequest $request, Incident $incident)
    {
        $incident         = $incident->presenter();
        $incident['code'] = 2002;
        return response()->json($incident)
                         ->setStatusCode(200, 'CREATE_SUCCESS');

    }

    /**
     * Create new incident.
     *
     * @param Request $request
     *
     * @return json
     */
    public function store(IncidentAdminApiRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.api');
            $incident          = $this->repository->create($attributes);
            $incident          = $incident->presenter();
            $incident['code']  = 2004;

            return response()->json($incident)
                             ->setStatusCode(201, 'STORE_SUCCESS');
        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4004,
            ])->setStatusCode(400, 'STORE_ERROR');
;
        }
    }

    /**
     * Show incident for editing.
     *
     * @param Request $request
     * @param Model   $incident
     *
     * @return json
     */
    public function edit(IncidentAdminApiRequest $request, Incident $incident)
    {
        $incident         = $incident->presenter();
        $incident['code'] = 2003;
        return response()-> json($incident)
                        ->setStatusCode(200, 'EDIT_SUCCESS');;
    }

    /**
     * Update the incident.
     *
     * @param Request $request
     * @param Model   $incident
     *
     * @return json
     */
    public function update(IncidentAdminApiRequest $request, Incident $incident)
    {
        try {

            $attributes = $request->all();

            $incident->update($attributes);
            $incident         = $incident->presenter();
            $incident['code'] = 2005;

            return response()->json($incident)
                             ->setStatusCode(201, 'UPDATE_SUCCESS');


        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4005,
            ])->setStatusCode(400, 'UPDATE_ERROR');

        }
    }

    /**
     * Remove the incident.
     *
     * @param Request $request
     * @param Model   $incident
     *
     * @return json
     */
    public function destroy(IncidentAdminApiRequest $request, Incident $incident)
    {

        try {

            $t = $incident->delete();

            return response()->json([
                'message'  => trans('messages.success.delete', ['Module' => trans('incident::incident.name')]),
                'code'     => 2006
            ])->setStatusCode(202, 'DESTROY_SUCCESS');

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4006,
            ])->setStatusCode(400, 'DESTROY_ERROR');
        }
    }
}
